@@ -140,45 +140,52 @@ namespace rapidfuzz {
 template <typename InputIt1, typename InputIt2>
 int64_t levenshtein_distance(InputIt1 first1, InputIt1 last1, InputIt2 first2, InputIt2 last2,
                              LevenshteinWeightTable weights = {1, 1, 1},
-                             int64_t score_cutoff = std::numeric_limits<int64_t>::max())
+                             int64_t score_cutoff = std::numeric_limits<int64_t>::max(),
+                             int64_t score_hint = std::numeric_limits<int64_t>::max())
 {
-    return detail::Levenshtein::distance(first1, last1, first2, last2, weights, score_cutoff);
+    return detail::Levenshtein::distance(first1, last1, first2, last2, weights, score_cutoff, score_hint);
 }
 
 template <typename Sentence1, typename Sentence2>
 int64_t levenshtein_distance(const Sentence1& s1, const Sentence2& s2,
                              LevenshteinWeightTable weights = {1, 1, 1},
-                             int64_t score_cutoff = std::numeric_limits<int64_t>::max())
+                             int64_t score_cutoff = std::numeric_limits<int64_t>::max(),
+                             int64_t score_hint = std::numeric_limits<int64_t>::max())
 {
-    return detail::Levenshtein::distance(s1, s2, weights, score_cutoff);
+    return detail::Levenshtein::distance(s1, s2, weights, score_cutoff, score_hint);
 }
 
 template <typename InputIt1, typename InputIt2>
 int64_t levenshtein_similarity(InputIt1 first1, InputIt1 last1, InputIt2 first2, InputIt2 last2,
-                               LevenshteinWeightTable weights = {1, 1, 1}, int64_t score_cutoff = 0.0)
+                               LevenshteinWeightTable weights = {1, 1, 1}, int64_t score_cutoff = 0,
+                               int64_t score_hint = 0)
 {
-    return detail::Levenshtein::similarity(first1, last1, first2, last2, weights, score_cutoff);
+    return detail::Levenshtein::similarity(first1, last1, first2, last2, weights, score_cutoff, score_hint);
 }
 
 template <typename Sentence1, typename Sentence2>
 int64_t levenshtein_similarity(const Sentence1& s1, const Sentence2& s2,
-                               LevenshteinWeightTable weights = {1, 1, 1}, int64_t score_cutoff = 0.0)
+                               LevenshteinWeightTable weights = {1, 1, 1}, int64_t score_cutoff = 0,
+                               int64_t score_hint = 0)
 {
-    return detail::Levenshtein::similarity(s1, s2, weights, score_cutoff);
+    return detail::Levenshtein::similarity(s1, s2, weights, score_cutoff, score_hint);
 }
 
 template <typename InputIt1, typename InputIt2>
 double levenshtein_normalized_distance(InputIt1 first1, InputIt1 last1, InputIt2 first2, InputIt2 last2,
-                                       LevenshteinWeightTable weights = {1, 1, 1}, double score_cutoff = 1.0)
+                                       LevenshteinWeightTable weights = {1, 1, 1}, double score_cutoff = 1.0,
+                                       double score_hint = 1.0)
 {
-    return detail::Levenshtein::normalized_distance(first1, last1, first2, last2, weights, score_cutoff);
+    return detail::Levenshtein::normalized_distance(first1, last1, first2, last2, weights, score_cutoff,
+                                                    score_hint);
 }
 
 template <typename Sentence1, typename Sentence2>
 double levenshtein_normalized_distance(const Sentence1& s1, const Sentence2& s2,
-                                       LevenshteinWeightTable weights = {1, 1, 1}, double score_cutoff = 1.0)
+                                       LevenshteinWeightTable weights = {1, 1, 1}, double score_cutoff = 1.0,
+                                       double score_hint = 1.0)
 {
-    return detail::Levenshtein::normalized_distance(s1, s2, weights, score_cutoff);
+    return detail::Levenshtein::normalized_distance(s1, s2, weights, score_cutoff, score_hint);
 }
 
 /**
@@ -243,17 +250,18 @@ double levenshtein_normalized_distance(const Sentence1& s1, const Sentence2& s2,
 template <typename InputIt1, typename InputIt2>
 double levenshtein_normalized_similarity(InputIt1 first1, InputIt1 last1, InputIt2 first2, InputIt2 last2,
                                          LevenshteinWeightTable weights = {1, 1, 1},
-                                         double score_cutoff = 0.0)
+                                         double score_cutoff = 0.0, double score_hint = 0.0)
 {
-    return detail::Levenshtein::normalized_similarity(first1, last1, first2, last2, weights, score_cutoff);
+    return detail::Levenshtein::normalized_similarity(first1, last1, first2, last2, weights, score_cutoff,
+                                                      score_hint);
 }
 
 template <typename Sentence1, typename Sentence2>
 double levenshtein_normalized_similarity(const Sentence1& s1, const Sentence2& s2,
                                          LevenshteinWeightTable weights = {1, 1, 1},
-                                         double score_cutoff = 0.0)
+                                         double score_cutoff = 0.0, double score_hint = 0.0)
 {
-    return detail::Levenshtein::normalized_similarity(s1, s2, weights, score_cutoff);
+    return detail::Levenshtein::normalized_similarity(s1, s2, weights, score_cutoff, score_hint);
 }
 
 /**
@@ -286,8 +294,132 @@ Editops levenshtein_editops(const Sentence1& s1, const Sentence2& s2,
     return detail::levenshtein_editops(detail::Range(s1), detail::Range(s2), score_hint);
 }
 
+#ifdef RAPIDFUZZ_SIMD
+namespace experimental {
+template <int MaxLen>
+struct MultiLevenshtein : public detail::MultiDistanceBase<MultiLevenshtein<MaxLen>, int64_t, 0,
+                                                           std::numeric_limits<int64_t>::max()> {
+private:
+    friend detail::MultiDistanceBase<MultiLevenshtein<MaxLen>, int64_t, 0,
+                                     std::numeric_limits<int64_t>::max()>;
+    friend detail::MultiNormalizedMetricBase<MultiLevenshtein<MaxLen>>;
+
+    constexpr static size_t get_vec_size()
+    {
+#    ifdef RAPIDFUZZ_AVX2
+        using namespace detail::simd_avx2;
+#    else
+        using namespace detail::simd_sse2;
+#    endif
+        if constexpr (MaxLen <= 8)
+            return native_simd<uint8_t>::size();
+        else if constexpr (MaxLen <= 16)
+            return native_simd<uint16_t>::size();
+        else if constexpr (MaxLen <= 32)
+            return native_simd<uint32_t>::size();
+        else if constexpr (MaxLen <= 64)
+            return native_simd<uint64_t>::size();
+
+        static_assert(MaxLen <= 64);
+    }
+
+    constexpr static size_t find_block_count(size_t count)
+    {
+        size_t vec_size = get_vec_size();
+        size_t simd_vec_count = detail::ceil_div(count, vec_size);
+        return detail::ceil_div(simd_vec_count * vec_size * MaxLen, 64);
+    }
+
+public:
+    MultiLevenshtein(size_t count, LevenshteinWeightTable aWeights = {1, 1, 1})
+        : input_count(count), PM(find_block_count(count) * 64), weights(aWeights)
+    {
+        str_lens.resize(result_count());
+        if (weights.delete_cost != 1 || weights.insert_cost != 1 || weights.replace_cost > 2)
+            throw std::invalid_argument("unsupported weights");
+    }
+
+    /**
+     * @brief get minimum size required for result vectors passed into
+     * - distance
+     * - similarity
+     * - normalized_distance
+     * - normalized_similarity
+     *
+     * @return minimum vector size
+     */
+    size_t result_count() const
+    {
+        size_t vec_size = get_vec_size();
+        size_t simd_vec_count = detail::ceil_div(input_count, vec_size);
+        return simd_vec_count * vec_size;
+    }
+
+    template <typename Sentence1>
+    void insert(const Sentence1& s1_)
+    {
+        insert(detail::to_begin(s1_), detail::to_end(s1_));
+    }
+
+    template <typename InputIt1>
+    void insert(InputIt1 first1, InputIt1 last1)
+    {
+        auto len = std::distance(first1, last1);
+        int block_pos = static_cast<int>((pos * MaxLen) % 64);
+        auto block = (pos * MaxLen) / 64;
+        assert(len <= MaxLen);
+
+        if (pos >= input_count) throw std::invalid_argument("out of bounds insert");
+
+        str_lens[pos] = static_cast<size_t>(len);
+        for (; first1 != last1; ++first1) {
+            PM.insert(block, *first1, block_pos);
+            block_pos++;
+        }
+        pos++;
+    }
+
+private:
+    template <typename InputIt2>
+    void _distance(int64_t* scores, size_t score_count, detail::Range<InputIt2> s2,
+                   int64_t score_cutoff = std::numeric_limits<int64_t>::max()) const
+    {
+        if (score_count < result_count())
+            throw std::invalid_argument("scores has to have >= result_count() elements");
+
+        if constexpr (MaxLen == 8)
+            detail::levenshtein_hyrroe2003_simd<uint8_t>(scores, PM, str_lens, s2, score_cutoff);
+        else if constexpr (MaxLen == 16)
+            detail::levenshtein_hyrroe2003_simd<uint16_t>(scores, PM, str_lens, s2, score_cutoff);
+        else if constexpr (MaxLen == 32)
+            detail::levenshtein_hyrroe2003_simd<uint32_t>(scores, PM, str_lens, s2, score_cutoff);
+        else if constexpr (MaxLen == 64)
+            detail::levenshtein_hyrroe2003_simd<uint64_t>(scores, PM, str_lens, s2, score_cutoff);
+    }
+
+    template <typename InputIt2>
+    int64_t maximum(size_t s1_idx, detail::Range<InputIt2> s2) const
+    {
+        return detail::levenshtein_maximum(static_cast<ptrdiff_t>(str_lens[s1_idx]), s2.size(), weights);
+    }
+
+    size_t get_input_count() const noexcept
+    {
+        return input_count;
+    }
+
+    size_t input_count;
+    size_t pos = 0;
+    detail::BlockPatternMatchVector PM;
+    std::vector<size_t> str_lens;
+    LevenshteinWeightTable weights;
+};
+} /* namespace experimental */
+#endif
+
 template <typename CharT1>
-struct CachedLevenshtein : public detail::CachedDistanceBase<CachedLevenshtein<CharT1>> {
+struct CachedLevenshtein : public detail::CachedDistanceBase<CachedLevenshtein<CharT1>, int64_t, 0,
+                                                             std::numeric_limits<int64_t>::max()> {
     template <typename Sentence1>
     explicit CachedLevenshtein(const Sentence1& s1_, LevenshteinWeightTable aWeights = {1, 1, 1})
         : CachedLevenshtein(detail::to_begin(s1_), detail::to_end(s1_), aWeights)
@@ -299,17 +431,18 @@ struct CachedLevenshtein : public detail::CachedDistanceBase<CachedLevenshtein<C
     {}
 
 private:
-    friend detail::CachedDistanceBase<CachedLevenshtein<CharT1>>;
+    friend detail::CachedDistanceBase<CachedLevenshtein<CharT1>, int64_t, 0,
+                                      std::numeric_limits<int64_t>::max()>;
     friend detail::CachedNormalizedMetricBase<CachedLevenshtein<CharT1>>;
 
     template <typename InputIt2>
     int64_t maximum(detail::Range<InputIt2> s2) const
     {
-        return detail::levenshtein_maximum(detail::Range(s1), s2, weights);
+        return detail::levenshtein_maximum(static_cast<ptrdiff_t>(s1.size()), s2.size(), weights);
     }
 
     template <typename InputIt2>
-    int64_t _distance(detail::Range<InputIt2> s2, int64_t score_cutoff) const
+    int64_t _distance(detail::Range<InputIt2> s2, int64_t score_cutoff, int64_t score_hint) const
     {
         if (weights.insert_cost == weights.delete_cost) {
             /* when insertions + deletions operations are free there can not be any edit distance */
@@ -318,8 +451,10 @@ private:
             /* uniform Levenshtein multiplied with the common factor */
             if (weights.insert_cost == weights.replace_cost) {
                 // max can make use of the common divisor of the three weights
-                int64_t new_max = detail::ceil_div(score_cutoff, weights.insert_cost);
-                int64_t dist = detail::uniform_levenshtein_distance(PM, detail::Range(s1), s2, new_max);
+                int64_t new_score_cutoff = detail::ceil_div(score_cutoff, weights.insert_cost);
+                int64_t new_score_hint = detail::ceil_div(score_hint, weights.insert_cost);
+                int64_t dist = detail::uniform_levenshtein_distance(PM, detail::Range(s1), s2,
+                                                                    new_score_cutoff, new_score_hint);
                 dist *= weights.insert_cost;
 
                 return (dist <= score_cutoff) ? dist : score_cutoff + 1;
